#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <popt.h>
#include <libharkio3.h>

#define ZDIR_TF          "transferFunction/"       //required
#define ZPATH_README     ZDIR_TF "README"          //optional
#define ZPATH_WHATISTHIS ZDIR_TF "whatisthis.txt"  //required
#define ZPATH_MIC        ZDIR_TF "microphones.xml" //required
#define ZPATH_SRC        ZDIR_TF "source.xml"      //required
#define ZDIR_TFLOC       ZDIR_TF "localization/"   //optional
#define ZDIR_TFSEP       ZDIR_TF "separation/"     //optional
#define ZPATH_TFLOC_PREF ZDIR_TFLOC "tf"           //either
#define ZPATH_TFSEP_PREF ZDIR_TFSEP "tf"           //either

#define ZFILENUM_REQUIRED  6
#define ZFILENUM_OPTIONAL  1

bool hasParsedOpt = false;
void help(poptContext con, int exitCode) {
    if (hasParsedOpt)
        return;

    printf("\n");
    printf("\n");
    printf("NAME:\n");
    printf("  harktoolcli-validtf\n");
    printf("\n");
    printf("Description:\n");
    printf("  Consistency check of the transfer function\n");
    printf("\n");
    poptPrintHelp(con, stdout, 0);
    printf("\n");
    printf("Output:\n");
    printf("  If a tf_file is valid, the command returns 0.\n");
    printf("\n");
    printf("\n");
    exit(exitCode);
}

int main(int argc, char **argv) {

    int errorp;
    struct zip *handle;

    int posCnt;
    int isShow = false;
    int isJson = false;
    char* logLevel = "E";

    struct poptOption optionsTable[] = {
    /*{ "longname", "shortname",
     *  argInfo,
     *  *arg, retval,
     *  description, argment description} */
    { "show", 's',
      POPT_ARG_NONE,
      &isShow, 's',
      "Show details", NULL },
    { "json", 'j',
      POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN,
      &isJson, 'j',
      "Show details (JSON)", NULL },
    { "logLevel", '\0',
      POPT_ARG_STRING | POPT_ARGFLAG_SHOW_DEFAULT,
      &logLevel, '-',
      "Log information level", "E|W|I|D" },
    { "help", '\0',
      0,
      NULL, '?',
      "Show this help message", NULL },
    POPT_TABLEEND };

    poptContext optCon = poptGetContext(NULL, argc, (const char**)argv, optionsTable, 0);
    poptSetOtherOptionHelp(optCon, "[OPTIONS]* tf_filename(zip)");

    // argc check
    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error, "argument error!!");
        help(optCon, EXIT_FAILURE);
    }

    // option processing
    int c;
    while ((c = poptGetNextOpt(optCon)) >= 0) {
        switch (c) {
        case '-':
            if (strcmp(logLevel, "W") == 0) {
                harkio_Log_setLevel(harkio_Log_Warning);
            } else if (strcmp(logLevel, "I") == 0) {
                harkio_Log_setLevel(harkio_Log_Info);
            } else if (strcmp(logLevel, "D") == 0) {
                harkio_Log_setLevel(harkio_Log_Debug);
            } else {
                harkio_Log_setLevel(harkio_Log_Error);
            }
            hasParsedOpt = true;
            break;
        case '?':
            // help
            help(optCon, EXIT_SUCCESS);
            /* no break */
        default:
            hasParsedOpt = true;
        }
    }
    if (c < -1) {
        /* an error occurred during option processing */
        harkio_Log_printf(harkio_Log_Error, "%s: %s",
                poptBadOption(optCon, POPT_BADOPTION_NOALIAS), poptStrerror(c));
        exit(EXIT_FAILURE);
    }

    harkio_Log_printf(harkio_Log_Debug, "-s=%s", (isShow) ? "true" : "false");

    // get filename
    const char *filename = poptGetArg(optCon);

    if (!filename) {
        harkio_Log_printf(harkio_Log_Error , "missing tf_filename");
        exit(EXIT_FAILURE);
    }

    // validate tf file
    handle = zip_open(filename, ZIP_CHECKCONS, &errorp);

    if (handle == NULL) {
        harkio_Log_printf(harkio_Log_Error, "can not open zip file [%s]", filename);
        exit(EXIT_FAILURE);
    }

    bool isTFValid = true;
    if (zip_name_locate(handle, ZDIR_TF, 0) == -1) {
        harkio_Log_printf(harkio_Log_Error , "%s not found.", ZDIR_TF);
        isTFValid = false;
    }
    if (zip_name_locate(handle, ZPATH_WHATISTHIS, 0) == -1) {
        harkio_Log_printf(harkio_Log_Error , "%s not found.", ZPATH_WHATISTHIS);
        isTFValid = false;
    }
    if (zip_name_locate(handle, ZPATH_MIC, 0) == -1) {
        harkio_Log_printf(harkio_Log_Error , "%s not found.", ZPATH_MIC);
        isTFValid = false;
    }
    if (zip_name_locate(handle, ZPATH_SRC, 0) == -1) {
        harkio_Log_printf(harkio_Log_Error , "%s not found.", ZPATH_SRC);
        isTFValid = false;
    }

    zip_close(handle);
    handle = NULL;

    if (!isTFValid) {
        return (EXIT_FAILURE);
    }

    //get position size
    harkio_TransferFunction *tf = NULL;

    tf = harkio_TransferFunction_fromFile(filename, NULL);
    if (tf == NULL) {
        harkio_Log_printf(harkio_Log_Error, "zip read file error");
        exit(EXIT_FAILURE);
    }
    posCnt = tf->poses->size;

    if (isShow) {
        fprintf(stdout, "Transfer Function Informations\n");
        // number of mics
        fprintf(stdout, " Microphones       : %d\n", tf->mics->size);
        // number of tf directions
        fprintf(stdout, " TF Directions     : %d\n", tf->poses->size);
        // contents of tf
        char* contents;
        if (tf->loctfs && tf->septfs)
            contents = "localization, separation";
        else if (tf->loctfs)
            contents = "localization";
        else if (tf->septfs)
            contents = "separation";
        else
            contents = "no tf matrix";
        fprintf(stdout, " Contents          : %s\n", contents);

        fprintf(stdout, "Configuration Informations\n");
        // config:comment
        fprintf(stdout, " Comment           : %s\n",
                (tf->cfg->comment) ? tf->cfg->comment : "");
        // SynchronousAverage
        fprintf(stdout, " SynchronousAverage: %d\n",
                tf->cfg->synchronousAverage);
        // TSPpath
        fprintf(stdout, " TSPpath           : %s\n",
                (tf->cfg->TSPpath) ? tf->cfg->TSPpath : "");
        // TSPOffset
        fprintf(stdout, " TSPOffset         : %d\n", tf->cfg->TSPoffset);
        // PeakSearch from to
        fprintf(stdout, " PeakSearch        : from %d to %d\n",
                tf->cfg->peakSearchFrom, tf->cfg->peakSearchTo);
        // nfft
        fprintf(stdout, " nfft              : %d\n", tf->cfg->nfft);
        // samplingRate
        fprintf(stdout, " samplingRate      : %d\n", tf->cfg->samplingRate);
        // signalMax
        fprintf(stdout, " signalMax         : %d\n", tf->cfg->signalMax);
        // TSPLength
        fprintf(stdout, " TSPLength         : %d\n", tf->cfg->TSPLength);
    } else if (isJson) {
        fprintf(stdout, "{\n    \"TFInfo\": {\n");
        // number of mics
        fprintf(stdout, "        \"Microphones\": %d ,\n", tf->mics->size);
        // number of tf directions
        fprintf(stdout, "        \"Directions\": %d ,\n", tf->poses->size);
        // contents of tf
        char* contents;
        if (tf->loctfs && tf->septfs)
            contents = "[ \"localization\" , \"separation\" ]";
        else if (tf->loctfs)
            contents = "[ \"localization\" ]";
        else if (tf->septfs)
            contents = "[ \"separation\" ]";
        else
            contents = "[]";
        fprintf(stdout, "        \"Contents\": %s\n    } ,\n", contents);

        fprintf(stdout, "    \"Config\": {\n");
        // config:comment
        fprintf(stdout, "        \"Comment\": \"%s\" ,\n",
                (tf->cfg->comment) ? tf->cfg->comment : "");
        // SynchronousAverage
        fprintf(stdout, "        \"SynchronousAverage\": %d ,\n",
                tf->cfg->synchronousAverage);
        // TSPpath
        fprintf(stdout, "        \"TSPpath\": \"%s\" ,\n",
                (tf->cfg->TSPpath) ? tf->cfg->TSPpath : "");
        // TSPOffset
        fprintf(stdout, "        \"TSPOffset\": %d ,\n", tf->cfg->TSPoffset);
        // PeakSearch from to
        fprintf(stdout, "        \"PeakSearch\": [ { \"from\" : %d, \"to\" : %d } ] ,\n",
                tf->cfg->peakSearchFrom, tf->cfg->peakSearchTo);
        // nfft
        fprintf(stdout, "        \"nfft\": %d ,\n", tf->cfg->nfft);
        // samplingRate
        fprintf(stdout, "        \"samplingRate\": %d ,\n", tf->cfg->samplingRate);
        // signalMax
        fprintf(stdout, "        \"signalMax\": %d ,\n", tf->cfg->signalMax);
        // TSPLength
        fprintf(stdout, "        \"TSPLength\": %d\n    }\n}", tf->cfg->TSPLength);
    }

    if (tf->loctfs && tf->internal_locsize != posCnt) {
        harkio_Log_printf(harkio_Log_Error, "localization matrix size(%d) differ from position size(%d)", tf->internal_locsize, posCnt);
        isTFValid = false;
    }
    if (tf->septfs && tf->internal_sepsize != posCnt) {
        harkio_Log_printf(harkio_Log_Error, "separation matrix size(%d) differ from position size(%d)", tf->internal_sepsize, posCnt);
        isTFValid = false;
    }

    harkio_TransferFunction_delete(&tf);

    // free poptContext
    poptFreeContext(optCon);

    if (isTFValid) {
        fprintf(stderr, "SUCCESS\n"); // "Successfully validated."
        return EXIT_SUCCESS;
    } else {
        return EXIT_FAILURE;
    }
}

